/* $Id: errmsgwin.cpp $ */
/** @file
 * IPRT - Status code messages.
 */

/*
 * Copyright (C) 2006-2020 Oracle Corporation
 *
 * This file is part of VirtualBox Open Source Edition (OSE), as
 * available from http://www.virtualbox.org. This file is free software;
 * you can redistribute it and/or modify it under the terms of the GNU
 * General Public License (GPL) as published by the Free Software
 * Foundation, in version 2 as it comes in the "COPYING" file of the
 * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
 * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
 *
 * The contents of this file may alternatively be used under the terms
 * of the Common Development and Distribution License Version 1.0
 * (CDDL) only, as it comes in the "COPYING.CDDL" file of the
 * VirtualBox OSE distribution, in which case the provisions of the
 * CDDL are applicable instead of those of the GPL.
 *
 * You may elect to license modified versions of this file under the
 * terms and conditions of either the GPL or the CDDL or both.
 */


/*********************************************************************************************************************************
*   Header Files                                                                                                                 *
*********************************************************************************************************************************/
#include <iprt/win/windows.h>

#include <iprt/errcore.h>
#include <iprt/asm.h>
#include <iprt/string.h>
#include <iprt/errcore.h>


/*********************************************************************************************************************************
*   Global Variables                                                                                                             *
*********************************************************************************************************************************/
/** Array of messages.
 * The data is generated by a sed script.
 */
static const RTWINERRMSG  g_aStatusMsgs[] =
{
#if !defined(IPRT_NO_ERROR_DATA) && !defined(DOXYGEN_RUNNING)
# include "errmsgcomdata.h"
# if defined(VBOX) && !defined(IN_GUEST)
#  include "errmsgvboxcomdata.h"
# endif

/* A few hardcoded items not in winerror.h */
# define HARDCODED_ENTRY(a_Name, aValue) { #a_Name, #a_Name, aValue }
    HARDCODED_ENTRY(AUDCLNT_E_NOT_INITIALIZED              , MAKE_HRESULT(SEVERITY_ERROR, 2185, 0x01)),
    HARDCODED_ENTRY(AUDCLNT_E_ALREADY_INITIALIZED          , MAKE_HRESULT(SEVERITY_ERROR, 2185, 0x02)),
    HARDCODED_ENTRY(AUDCLNT_E_WRONG_ENDPOINT_TYPE          , MAKE_HRESULT(SEVERITY_ERROR, 2185, 0x03)),
    HARDCODED_ENTRY(AUDCLNT_E_DEVICE_INVALIDATED           , MAKE_HRESULT(SEVERITY_ERROR, 2185, 0x04)),
    HARDCODED_ENTRY(AUDCLNT_E_NOT_STOPPED                  , MAKE_HRESULT(SEVERITY_ERROR, 2185, 0x05)),
    HARDCODED_ENTRY(AUDCLNT_E_BUFFER_TOO_LARGE             , MAKE_HRESULT(SEVERITY_ERROR, 2185, 0x06)),
    HARDCODED_ENTRY(AUDCLNT_E_OUT_OF_ORDER                 , MAKE_HRESULT(SEVERITY_ERROR, 2185, 0x07)),
    HARDCODED_ENTRY(AUDCLNT_E_UNSUPPORTED_FORMAT           , MAKE_HRESULT(SEVERITY_ERROR, 2185, 0x08)),
    HARDCODED_ENTRY(AUDCLNT_E_INVALID_SIZE                 , MAKE_HRESULT(SEVERITY_ERROR, 2185, 0x09)),
    HARDCODED_ENTRY(AUDCLNT_E_DEVICE_IN_USE                , MAKE_HRESULT(SEVERITY_ERROR, 2185, 0x0a)),
    HARDCODED_ENTRY(AUDCLNT_E_BUFFER_OPERATION_PENDING     , MAKE_HRESULT(SEVERITY_ERROR, 2185, 0x0b)),
    HARDCODED_ENTRY(AUDCLNT_E_THREAD_NOT_REGISTERED        , MAKE_HRESULT(SEVERITY_ERROR, 2185, 0x0c)),
    HARDCODED_ENTRY(AUDCLNT_E_EXCLUSIVE_MODE_NOT_ALLOWED   , MAKE_HRESULT(SEVERITY_ERROR, 2185, 0x0e)),
    HARDCODED_ENTRY(AUDCLNT_E_ENDPOINT_CREATE_FAILED       , MAKE_HRESULT(SEVERITY_ERROR, 2185, 0x0f)),
    HARDCODED_ENTRY(AUDCLNT_E_SERVICE_NOT_RUNNING          , MAKE_HRESULT(SEVERITY_ERROR, 2185, 0x10)),
    HARDCODED_ENTRY(AUDCLNT_E_EVENTHANDLE_NOT_EXPECTED     , MAKE_HRESULT(SEVERITY_ERROR, 2185, 0x11)),
    HARDCODED_ENTRY(AUDCLNT_E_EXCLUSIVE_MODE_ONLY          , MAKE_HRESULT(SEVERITY_ERROR, 2185, 0x12)),
    HARDCODED_ENTRY(AUDCLNT_E_BUFDURATION_PERIOD_NOT_EQUAL , MAKE_HRESULT(SEVERITY_ERROR, 2185, 0x13)),
    HARDCODED_ENTRY(AUDCLNT_E_EVENTHANDLE_NOT_SET          , MAKE_HRESULT(SEVERITY_ERROR, 2185, 0x14)),
    HARDCODED_ENTRY(AUDCLNT_E_INCORRECT_BUFFER_SIZE        , MAKE_HRESULT(SEVERITY_ERROR, 2185, 0x15)),
    HARDCODED_ENTRY(AUDCLNT_E_BUFFER_SIZE_ERROR            , MAKE_HRESULT(SEVERITY_ERROR, 2185, 0x16)),
    HARDCODED_ENTRY(AUDCLNT_E_CPUUSAGE_EXCEEDED            , MAKE_HRESULT(SEVERITY_ERROR, 2185, 0x17)),
    HARDCODED_ENTRY(AUDCLNT_E_BUFFER_ERROR                 , MAKE_HRESULT(SEVERITY_ERROR, 2185, 0x18)),
    HARDCODED_ENTRY(AUDCLNT_E_BUFFER_SIZE_NOT_ALIGNED      , MAKE_HRESULT(SEVERITY_ERROR, 2185, 0x19)),
    HARDCODED_ENTRY(AUDCLNT_E_INVALID_DEVICE_PERIOD        , MAKE_HRESULT(SEVERITY_ERROR, 2185, 0x20)),
    HARDCODED_ENTRY(AUDCLNT_E_INVALID_STREAM_FLAG          , MAKE_HRESULT(SEVERITY_ERROR, 2185, 0x21)),
    HARDCODED_ENTRY(AUDCLNT_E_ENDPOINT_OFFLOAD_NOT_CAPABLE , MAKE_HRESULT(SEVERITY_ERROR, 2185, 0x22)),
    HARDCODED_ENTRY(AUDCLNT_E_OUT_OF_OFFLOAD_RESOURCES     , MAKE_HRESULT(SEVERITY_ERROR, 2185, 0x23)),
    HARDCODED_ENTRY(AUDCLNT_E_OFFLOAD_MODE_ONLY            , MAKE_HRESULT(SEVERITY_ERROR, 2185, 0x24)),
    HARDCODED_ENTRY(AUDCLNT_E_NONOFFLOAD_MODE_ONLY         , MAKE_HRESULT(SEVERITY_ERROR, 2185, 0x25)),
    HARDCODED_ENTRY(AUDCLNT_E_RESOURCES_INVALIDATED        , MAKE_HRESULT(SEVERITY_ERROR, 2185, 0x26)),
    HARDCODED_ENTRY(AUDCLNT_E_RAW_MODE_UNSUPPORTED         , MAKE_HRESULT(SEVERITY_ERROR, 2185, 0x27)),
    HARDCODED_ENTRY(AUDCLNT_E_ENGINE_PERIODICITY_LOCKED    , MAKE_HRESULT(SEVERITY_ERROR, 2185, 0x28)),
    HARDCODED_ENTRY(AUDCLNT_E_ENGINE_FORMAT_LOCKED         , MAKE_HRESULT(SEVERITY_ERROR, 2185, 0x29)),
    HARDCODED_ENTRY(AUDCLNT_E_HEADTRACKING_ENABLED         , MAKE_HRESULT(SEVERITY_ERROR, 2185, 0x30)),
    HARDCODED_ENTRY(AUDCLNT_E_HEADTRACKING_UNSUPPORTED     , MAKE_HRESULT(SEVERITY_ERROR, 2185, 0x40)),
    HARDCODED_ENTRY(AUDCLNT_S_BUFFER_EMPTY                 , MAKE_SCODE(SEVERITY_SUCCESS, 2185,     1)),
    HARDCODED_ENTRY(AUDCLNT_S_THREAD_ALREADY_REGISTERED    , MAKE_SCODE(SEVERITY_SUCCESS, 2185,     2)),
    HARDCODED_ENTRY(AUDCLNT_S_POSITION_STALLED             , MAKE_SCODE(SEVERITY_SUCCESS, 2185,     3)),
# undef HARDCODED_ENTRY
#else
    { "Success.", "ERROR_SUCCESS", 0 },
#endif
    { NULL, NULL, 0 }
};


/** Temporary buffers to format unknown messages in.
 * @{
 */
static char                 g_aszUnknownStr[4][64];
static RTWINERRMSG          g_aUnknownMsgs[4] =
{
    { &g_aszUnknownStr[0][0], &g_aszUnknownStr[0][0], 0 },
    { &g_aszUnknownStr[1][0], &g_aszUnknownStr[1][0], 0 },
    { &g_aszUnknownStr[2][0], &g_aszUnknownStr[2][0], 0 },
    { &g_aszUnknownStr[3][0], &g_aszUnknownStr[3][0], 0 }
};
/** Last used index in g_aUnknownMsgs. */
static volatile uint32_t    g_iUnknownMsgs;
/** @} */


/**
 * Get the message corresponding to a given status code.
 *
 * @returns Pointer to read-only message description.
 * @param   rc      The status code.
 */
RTDECL(PCRTWINERRMSG) RTErrWinGet(long rc)
{
    unsigned i;
    for (i = 0; i < RT_ELEMENTS(g_aStatusMsgs) - 1U; i++)
        if (g_aStatusMsgs[i].iCode == rc)
            return &g_aStatusMsgs[i];

    /* The g_aStatusMsgs table contains a wild mix of error codes with and
     * without included facility and severity. So the chance is high that there
     * was no exact match. Try to find a non-exact match, and include the
     * actual value in case we pick the wrong entry. Better than always using
     * the "Unknown Status" case. */
    for (i = 0; i < RT_ELEMENTS(g_aStatusMsgs) - 1U; i++)
        if (g_aStatusMsgs[i].iCode == HRESULT_CODE(rc))
        {
            int32_t iMsg = (ASMAtomicIncU32(&g_iUnknownMsgs) - 1) % RT_ELEMENTS(g_aUnknownMsgs);
            RTStrPrintf(&g_aszUnknownStr[iMsg][0], sizeof(g_aszUnknownStr[iMsg]), "%s 0x%X", g_aStatusMsgs[i].pszDefine, rc);
            return &g_aUnknownMsgs[iMsg];
        }

    /*
     * Need to use the temporary stuff.
     */
    int32_t iMsg = (ASMAtomicIncU32(&g_iUnknownMsgs) - 1) % RT_ELEMENTS(g_aUnknownMsgs);
    RTStrPrintf(&g_aszUnknownStr[iMsg][0], sizeof(g_aszUnknownStr[iMsg]), "Unknown Status 0x%X", rc);
    return &g_aUnknownMsgs[iMsg];
}


RTDECL(PCRTCOMERRMSG) RTErrCOMGet(uint32_t rc)
{
    return RTErrWinGet((long)rc);
}

